<xsl:transform version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:my="http://xml.com/my-template-language"
>

<xsl:param name="template-uri" />

<xsl:variable name="template" select="document($template-uri)" />
<xsl:variable name="source" select="/" />

<xsl:template match="/">
	<xsl:apply-templates select="$template/node()" />
</xsl:template>

<xsl:template match="*">
	<xsl:param name="current-album" />
	<xsl:param name="current-artist" />
	<xsl:element name="{name()}">
		<xsl:apply-templates select="@* | node()">
			<xsl:with-param name="current-album" select="$current-album" />
			<xsl:with-param name="current-artist" select="$current-artist" />
		</xsl:apply-templates>
	</xsl:element>
</xsl:template>

<xsl:template match="@*">
	<xsl:attribute name="{name()}">
		<xsl:value-of select="." />
	</xsl:attribute>
</xsl:template>

<xsl:template match="text()">
	<xsl:if test="normalize-space()">
		<xsl:value-of select="." />
	</xsl:if>
</xsl:template>

<xsl:template match="my:text">
	<xsl:value-of select="." />
</xsl:template>

<xsl:template match="my:owner-name">
	<xsl:value-of select="$source/collection/owner/name" />
</xsl:template>

<xsl:template match="my:owner-email">
	<xsl:value-of select="$source/collection/owner/email" />
</xsl:template>

<xsl:template match="my:if-owner-has-email">
	<xsl:if test="$source/collection/owner/email">
		<xsl:apply-templates />
	</xsl:if>
</xsl:template>

<xsl:template match="my:for-each-album">
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album">
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-album" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

<xsl:template match="my:album-artist">
	<xsl:param name="current-album" />
	<xsl:value-of select="$current-album/artist" />
</xsl:template>

<xsl:template match="my:album-title">
	<xsl:param name="current-album" />
	<xsl:value-of select="$current-album/title" />
</xsl:template>

<xsl:template match="my:for-each-artist">
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album[not(artist = preceding-sibling::album/artist)]/artist">
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-artist" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

<xsl:template match="my:artist-name">
	<xsl:param name="current-artist" />
	<xsl:value-of select="$current-artist" />
</xsl:template>

<xsl:template match="my:for-each-album-by-artist">
	<xsl:param name="current-artist" />
	<xsl:variable name="current" select="." />
	<xsl:for-each select="$source/collection/album[artist = $current-artist]">
		<xsl:apply-templates select="$current/node()">
			<xsl:with-param name="current-artist" select="$current-artist" />
			<xsl:with-param name="current-album" select="." />
		</xsl:apply-templates>
	</xsl:for-each>
</xsl:template>

</xsl:transform>
